/*
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>
#include <errno.h>
#include <glib.h>
#include <sys/mman.h>
#include <tizen.h>
#include <dlog.h>
#include <wayland-client.h>
#include <Ecore_Wayland.h>
#include <Elementary.h>
#include <wayland-extension/tizen-extension-client-protocol.h>
#include <tizen-remote-surface-client-protocol.h>
#include <aul_screen_connector.h>

#include "screen_connector_provider.h"

#ifdef LOG_TAG
#undef LOG_TAG
#endif

#define LOG_TAG "SC_PROVIDER"

static int __init_count;
static struct tizen_remote_surface_manager *__rsm;

struct __remote_surface_s {
	struct wl_surface *surface;
	struct tizen_remote_surface_provider *rsp;
	char *id;
	Ecore_Wl_Window *win;
};

static void __rsp_resource_id_cb(void *data, struct tizen_remote_surface_provider *provider, uint32_t res_id)
{
	LOGE("add app screen : %d", res_id);
	aul_screen_connector_add_app_screen(res_id);
}

static void __rsp_visibility_cb(void *data, struct tizen_remote_surface_provider *provider, uint32_t visibility)
{
	struct __remote_surface_s *remote = (struct __remote_surface_s *)data;
	Ecore_Wl_Event_Window_Visibility_Change *ev;

	ev = calloc(1, sizeof(Ecore_Wl_Event_Window_Visibility_Change));
	if (!ev) {
		LOGE("out of memory");
		return;
	}

	ev->win = ecore_wl_window_id_get(remote->win);

	if (visibility == TIZEN_REMOTE_SURFACE_VISIBILITY_TYPE_VISIBLE)
		ev->fully_obscured = 0;
	else
		ev->fully_obscured = 1;

	LOGD("visibility changed:%d", visibility);

	ecore_event_add(ECORE_WL_EVENT_WINDOW_VISIBILITY_CHANGE, ev, NULL, NULL);
}

static const struct tizen_remote_surface_provider_listener __rsp_listener = {
	__rsp_resource_id_cb,
	__rsp_visibility_cb,
};

EXPORT_API int screen_connector_provider_remote_enable(const char *id, struct wl_surface *surface)
{
	struct __remote_surface_s *remote;
	struct wl_display *display;

	if (!__rsm) {
		LOGE("__rsm is not ready");
		return -1;
	}

	if (!surface || !id) {
		LOGE("invalid parameter");
		return -1;
	}

	display = ecore_wl_display_get();
	if (!display) {
		LOGE("Fail to get");
		return -1;
	}

	remote = (struct __remote_surface_s *)malloc(sizeof(struct __remote_surface_s));
	if (!remote) {
		LOGE("out of memory");
		return -1;
	}

	remote->surface = surface;
	remote->win = ecore_wl_window_surface_find(surface);
	if (!remote->win) {
		LOGE("failed to find win");
		free(remote);
		return -1;
	}

	remote->rsp = tizen_remote_surface_manager_create_provider(__rsm, surface);
	if (!remote->rsp) {
		LOGE("failed to create provider");
		free(remote);
		return -1;
	}

	remote->id = strdup(id);
	if (!remote->id) {
		free(remote);
		LOGE("out of memory");
		return -1;
	}

	tizen_remote_surface_provider_add_listener(remote->rsp, &__rsp_listener, remote);

	wl_display_roundtrip(display);
	LOGD("%d surface remote enable");

	return 0;
}

EXPORT_API int screen_connector_provider_init(void)
{
	struct wl_registry *registry;
	Ecore_Wl_Global *global;
	Eina_Inlist *globals;

	if (__init_count > 0)
		return 0;

	ecore_wl_init(NULL);

	registry = ecore_wl_registry_get();
	globals = ecore_wl_globals_get();

	if (!registry || !globals) {
		LOGE("could not get registry(%p) or global list(%p)", registry,
				globals);
		return -1;
	}

	EINA_INLIST_FOREACH(globals, global) {
		if (!strcmp(global->interface, "tizen_remote_surface_manager")) {
			__rsm = wl_registry_bind(registry, global->id,
					&tizen_remote_surface_manager_interface, 1);
		}
	}

	if (!__rsm) {
		LOGE("could not get remote surface manager");
		return -1;
	}
	__init_count++;

	return 0;
}

EXPORT_API int screen_connector_provider_fini(void)
{
	if (__init_count > 1)
		return 0;

	if (__rsm)
		tizen_remote_surface_manager_destroy(__rsm);

	ecore_wl_shutdown();

	__init_count--;
	return 0;
}
